function [xh1,uh1]=fem_1d_evalsol(uh,geom,p,Ne,np)
% fem_1d_evalsol: evaluate the fem-solution provided by fem_1d_solve 
% on a finer grid xh1
% [xh1,uh1]=fem_1d_evalsol(uh,geom,p,Ne,np)
%
% Input: uh= numerical solution provided by fem_1d_solve
%        geom= struct with fields:
%          geom.xa, geom.xb = end-points of Omega
%        p = local polynomial degree (1 or 2)
%        Ne =  number of elements of the partition
%        np =  number of nodes (for each element) at which you want to
%        evaluate the numerical solution
%
% Output: xh1 = (column array) the new nodes
%         uh1 = (column array) the numerical solution evaluated at the nodes xh1 

%   Written by Paola Gervasio
%   $Date: 2020/04/29$

xe=linspace(geom.xa,geom.xb,Ne+1); % array of the end-points of the elements
h=(geom.xb-geom.xa)/Ne; % step-size
N=Ne*(np-1)+1; % total number of nodes (np in each element)
xh1=zeros(N,1); % array for the new nodes
uh1=zeros(N,1); % array for the new values of the solution

xnew=linspace(0,1,np)'; % new nodes on the reference interval

% evaluate the basis function on the reference interval (0,1)
if p==1
 % basis functions on the reference interval (0,1)
phi=[1-xnew, xnew];

elseif p==2
  % basis functions 
phi=[2*xnew.^2-3*xnew+1, -4*xnew.^2+4*xnew, 2*xnew.^2-xnew];
end

% connectivity matrix on the real mesh
conn=(1:p+1)'; nmax=p+1;
for ie=2:Ne
  conn=[conn,(nmax:nmax+p)'];
  nmax=nmax+p;
end
% connectivity matrix on the new mesh
conn1=(1:np)'; nmax=np;
for ie=2:Ne
  conn1=[conn1,(nmax:nmax+np-1)'];
  nmax=nmax+np-1;
end

% evaluate the solution at the new nodes
for ie=1:Ne
  xh1(conn1(:,ie))=xe(ie)+h*xnew;
  uh1(conn1(:,ie))=phi*uh(conn(:,ie));
end
  
